/*
 * Copyright (c) 2021 Huawei Device Co., Ltd.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef OHOS_AAFWK_INTENT_FILTER_H
#define OHOS_AAFWK_INTENT_FILTER_H

#include <string>
#include <vector>

#include "parcel.h"

namespace OHOS {
namespace AAFwk {

class Intent;
class IntentFilter final : public Parcelable {
public:
    IntentFilter();
    inline ~IntentFilter()
    {}

    std::string GetEntity() const;

    void SetEntity(const std::string &entity);

    void AddAction(const std::string &action);

    int CountAction() const;

    std::string GetAction(int index) const;

    void RemoveAction(const std::string &action);

    bool HasAction(const std::string &action) const;

    bool Match(const Intent &intent) const;

    virtual bool Marshalling(Parcel &parcel) const;

    static IntentFilter *Unmarshalling(Parcel &parcel);

private:
    bool ReadFromParcel(Parcel &parcel);
    bool MatchAction(const std::string &action) const;
    bool MatchEntity(const std::string &entity) const;

    std::string entity_;
    std::vector<std::string> actions_;
};

} // namespace AAFwk
} // namespace OHOS

#endif // OHOS_AAFWK_INTENT_FILTER_H
